#ifndef BOARD_H
#define BOARD_H

#include "assert.h"
#include "itype.h"
#include "sbr.h"
#include "bitboard.h"

#define BOARD_SQUARES   64

typedef struct BOARD {
   uint8 square[64];
   
   /* Bitboard encoding of the above data */
   /*  Index 0 is for pawns */
   /*  Index 1 is for knights */
   /*  Index 2 is for bishops */
   /*  Index 3 is for rooks */
   /*  Index 4 is for queens */
   /*  Index 5 is for kings */
   /*  Index 6 is for unmoved pieces */
   BITBOARD bbp[7];
   /*  Index 0 is for white pieces */
   /*  Index 1 is for black pieces */
   BITBOARD bbc[2];
   
   /* En-passant square */
   int ep_square;
} BOARD;

/* Symbolic constants: names of the squares on the chess board */
#define A1 0
#define B1 1
#define C1 2
#define D1 3
#define E1 4
#define F1 5
#define G1 6
#define H1 7
#define A2 8
#define B2 9
#define C2 10
#define D2 11
#define E2 12
#define F2 13
#define G2 14
#define H2 15
#define A3 16
#define B3 17
#define C3 18
#define D3 19
#define E3 20
#define F3 21
#define G3 22
#define H3 23
#define A4 24
#define B4 25
#define C4 26
#define D4 27
#define E4 28
#define F4 29
#define G4 30
#define H4 31
#define A5 32
#define B5 33
#define C5 34
#define D5 35
#define E5 36
#define F5 37
#define G5 38
#define H5 39
#define A6 40
#define B6 41
#define C6 42
#define D6 43
#define E6 44
#define F6 45
#define G6 46
#define H6 47
#define A7 48
#define B7 49
#define C7 50
#define D7 51
#define E7 52
#define F7 53
#define G7 54
#define H7 55
#define A8 56
#define B8 57
#define C8 58
#define D8 59
#define E8 60
#define F8 61
#define G8 62
#define H8 63
#define EX 0xFF

extern void clear_board(BOARD *board);
extern void begin_board(BOARD *board);

extern void calculate_bitboards(BOARD *board);
extern void print_bitboards(BOARD *board);

#include "pieces.h"

/* Functions used to manipulate a chess board - written as inlineble functions */
extern inline uint8 get_piece(const BOARD *board, const uint8 where)
{
   assert(where<=H8);
   return board->square[where];
}

extern inline void remove_piece(BOARD *board, const uint8 where)
{
   uint8 piece;
   
   assert(where<=H8);

   piece = board->square[where];
   unset_bitboard(&(board->bbc[(piece>>7)&1]), where);
   if (piece&INIBIT)
      unset_bitboard(&(board->bbp[6]), where);
   unset_bitboard(&(board->bbp[sbr(piece&PIECE)]), where);
   
   board->square[where] = NONE;
}

extern inline void place_piece(BOARD *board, const uint8 where, const uint8 piece)
{
   assert(where<=H8);

   remove_piece(board, where);
   set_bitboard(&(board->bbc[(piece>>7)&1]), where);
   if (piece&INIBIT)
      set_bitboard(&(board->bbp[6]), where);
   set_bitboard(&(board->bbp[sbr(piece&PIECE)]), where);
   
   board->square[where] = piece;
}
#endif
