#ifndef MOVEGEN_H
#define MOVEGEN_H

#include "board.h"
#include "move.h"

/* Move list */
typedef struct MOVELIST {
   MOVE move[MAXMOVES];    /* List of moves */
   int nummoves;           /* Number of moves in this list */
   int cmove;              /* Last move accessed, or nummoves if list traversed completely */
} MOVELIST;

typedef struct MOVE_GEN_STRUCT {
   MOVELIST total;
   MOVELIST capture;
   MOVELIST promotion;
   /*
   MOVELIST castle;
   MOVELIST check;
   */
   MOVELIST normal;
} MOVE_GEN_STRUCT;

extern void calculate_move_tables(void);
extern void print_move_list(MOVE_GEN_STRUCT *ml, int *nummoves, char ***movestring);

extern void move_gen_start(BOARD *board, int player, MOVE_GEN_STRUCT *ml);
extern int get_next_move(MOVE_GEN_STRUCT *ml, MOVE *m);

extern int is_valid_move(MOVE m, MOVE_GEN_STRUCT *ml);

extern int covers(BOARD *board, int player, int square);

/* returns TRUE if the player is being checked */
extern inline int check(BOARD *board, int player)
{
   int square;
   
   assert(board);
   
   square = sbr_longlong(and_bitboards(board->bbc[player>>7], board->bbp[5]));
   
   return covers(board, BLACK-player, square);
}

#endif
