#ifndef PIECES_H
#define PIECES_H

#include "itype.h"
#include "sbr.h"

#define NUM_PIECE_TYPES  6

/* Pieces and their symbolic names/bitfield identifiers */
#define NONE     0
#define PAWN     1
#define KNIGHT   2
#define BISHOP   4
#define ROOK     8
#define QUEEN    16
#define KING     32

/* INIBIT is 1 if a piece hasn't moved yet */
#define INIBIT   64

/* Bitfield colours */
#define WHITE    0
#define BLACK    128

/* Piece classes */
#define HEAVY     (ROOK|QUEEN)
#define LIGHT     (KNIGHT|BISHOP)
#define OFFICER   (HEAVY|LIGHT)
#define TIMBER    (OFFICER|PAWN)
#define PIECE     (TIMBER|KING)

/* Special move flags */
#define CASTLE_MOVE     (KING|ROOK|INIBIT)
#define ENPASSANT_MOVE  (PAWN|KING)
#define PROMOTION_MOVE  (PAWN|OFFICER)

/* Special evaluations */
#define CHECKMATE piece_values[5]
#define STALEMATE 0
#define INFINITY (2*CHECKMATE)

extern int piece_values[6];

/* Getting onformation about pieces */
extern inline uint8 piece_colour(uint8 piece)
{
   return piece&BLACK;
}

extern inline int piece_value(uint8 piece)
{
   return piece_values[sbr(piece&PIECE)];
}

#endif
