/* AGC2 Chess Programme */
/* Ugly hack put together from a separate chess game */
#include "IGame.h"

extern "C" {
#include "chsgfx.h"
#include "genrand.h"
#include "hashgen.h"
#include "hashtab.h"
#include "sbr.h"
#include "bitboard.h"
#include "board.h"
#include "pieces.h"
#include "move.h"
#include "game.h"
#include "evaluate.h"
#include "alphabet.h"
#include "compplay.h"
//#include "ponder.h"
}

#define SELECT_FREE     0
#define SELECT_ORIGIN   1
#define SELECT_DONE     2

static volatile int sec_counter;
static DATAFILE *chessgfx = NULL;
static BITMAP *board_bmp = NULL;
static GAMESTATE game;
static FONT *textfont;
static FONT *figfont;
static int sel_x = 0;
static int sel_y = 0;
static int from_x = 0;
static int from_y = 0;
static int to_x = 0;
static int to_y = 0;
static int select_state;
static int think_time;
static int max_time = 10;     /* Up to ten seconds, set to 0 to search full depth */
static int max_depth = 12;    /* Maximum depth to search */

static int move_input;
static int piece;
static int from;
static int to;
static int ptaken;
static int quit;
static int player;
static int redraw_screen;
static int old_x = 0;
static int old_y = 0;

static MOVE m;

static int display_prune_count;
static int display_depth_reached;
static int display_evaluate_count;

/* Places a piece on the board. The placement string is of the form Kf3 */
/* If colour is 0, the piece is assumed to be white, otherwise it is black */
static void blit_piece(BITMAP *board, int where, int piece)
{
   int colour;
   int c = 0;
   int x, y;
   
   assert(chessgfx);
   
   colour = piece & BLACK;
   
   switch (piece & (KING|QUEEN|ROOK|BISHOP|KNIGHT|PAWN)) {
      case KING:
         c = colour?BK_PCX:WK_PCX;
         break;
      case QUEEN:
         c = colour?BQ_PCX:WQ_PCX;
         break;
      case ROOK:
         c = colour?BR_PCX:WR_PCX;
         break;
      case BISHOP:
         c = colour?BB_PCX:WB_PCX;
         break;
      case KNIGHT:
         c = colour?BN_PCX:WN_PCX;
         break;
      case PAWN:
         c = colour?BP_PCX:WP_PCX;
         break;
   }

   x = where & 7;
   y = where >> 3;
   
   masked_blit((BITMAP *)chessgfx[c].dat, board, 0,0, 40*(x),40*(7-y), 40,40);
}

static void blit_position(BITMAP *bmp, BOARD board)
{
   int c;
   
   for (c=0; c<64; c++) {
      blit((BITMAP *)chessgfx[(c^(c>>3))&1?WS_PCX:BS_PCX].dat, bmp, 0,0, 40*(c&7),40*(7-(c>>3)), 40,40);
      if (get_piece(&board, c)) {
         blit_piece(bmp, c, get_piece(&board, c));
      }
   }
   
   /* Position indicator */
   masked_blit((BITMAP *)chessgfx[INDICATOR].dat, bmp, 0,0, 40*sel_x, 40*sel_y, 40, 40);

   if (select_state == SELECT_ORIGIN) {
      /* Position indicator */
      masked_blit((BITMAP *)chessgfx[INDICATOR].dat, bmp, 0,0, 40*from_x, 40*from_y, 40, 40);
   }
}

static void print_principle_variation(void)
{
   char s[32];
   int c;
   int x = 64;
   int y = 400;
   int w = 320;
   int h = 24;
   
   rectfill(screen, x,y, x+w, y+h, makecol(240, 220, 200));
   text_mode(makecol(240, 220, 200));

   for (c=0; c<game.pv.movecount; c++) {      
      textprintf(screen, figfont, 
                 x+4+70*c, y+4, 0, 
                 "%s", get_move_string_algebraic(NULL, game.pv.move[c], s));
   }

   text_mode(0);
}

static void draw_screen(BITMAP *bmp)
{
   char s[32];
   int nm;
   char **ms;
   int c;
   int x, y;
   int w, h;
   
   clear(bmp);

   blit_position(board_bmp, game.board);
   blit(board_bmp, bmp, 0,0, 64, 64, 320, 320);

   x = 16;
   y = 400;
   rectfill(bmp, x,y, 640, 480, 0);
   for (c=0; c<game.pv.movecount; c++) {      
      //printf("%s\t", ms[c]);
      text_mode(0);
      textprintf(bmp, figfont, x, y, makecol(255,255,255), get_move_string_algebraic(NULL, game.pv.move[c], s));
      x+=text_length(figfont, s)+8;
      if (x>560) {
         x = 16;
         y+=16;
      }
   }

   rectfill(bmp, 0,0, 640, 48, 0);
   textprintf(bmp, textfont, 0, 0, makecol(255,255,255), "%d pos. evaluated in %ds (%d pos./s). %d prunings, depth = %d", display_evaluate_count, think_time, display_evaluate_count/(think_time?think_time:1), display_prune_count, display_depth_reached);
   textprintf(bmp, textfont, 0, 16, makecol(255,255,255), "%+8d (%+8d)", evaluate(&game), game.eval);
   if (abs(game.eval) == CHECKMATE) {
      if (display_depth_reached == 0) {
         textprintf(bmp, textfont, 0, 32, makecol(255,255,255), "Mate");
      } else {
         textprintf(bmp, textfont, 0, 32, makecol(255,255,255), "Mate in %d", (display_depth_reached+1)/2);
      }
   }
   
   /* Principle variation */
   w = 48*8;
   h = 24;
   x = 16;
   y = 432;
   rectfill(bmp, x,y, x+w, y+h, makecol(240, 220, 200));

   for (c=0; c<game.pv.movecount; c++) {      
      textprintf_ex(bmp, figfont, 
                 x+4+70*c, y+4, 0, makecol(240, 220, 200),
                 "%s", get_move_string_algebraic(NULL, game.pv.move[c], s));
   }
}

static void inc_sec_counter(void)
{
   sec_counter++;
   /* Abort the evaluation if the timelimit is exceeded */
   if (sec_counter == max_time)
      abort_evaluation = 1;
}
END_OF_STATIC_FUNCTION(inc_sec_counter)


class ChessInterface : public IGame {
	public:
		ChessInterface() : IGame() {
		}

		~ChessInterface() {
		}

		char *GetName() {
			return "Simple Chess game";
		}

		char *GetAuthor() {
			return "Evert Glebbeek";
		}

		char *GetDescription() {
			return "A simple chess programme. The chess programme"
                               "itself is independent of the AMGC interface, so"
                               "this is really just a frontend to it.";
		}

		char *GetVersion() {
			return "0.9.0";
		}

		char *GetIconPath() {
			return "chessico.pcx";
		}

		bool Init() {
			char buf[1024];
                        if (!IGame::Init()) {
				return false;
			}

			// custom initialization goes here
			// ...
                        install_int(inc_sec_counter, 1000);
			
			get_executable_name(buf, 1024);
                        set_color_conversion(COLORCONV_TOTAL|COLORCONV_KEEP_TRANS);
			replace_filename(buf, buf, "games/chess/board.dat", 1024);
			chessgfx = load_datafile(buf);
                        if (!chessgfx) return false;
                        
                        figfont = (FONT *)chessgfx[FIGFONT].dat;
                        textfont = (FONT *)chessgfx[TEXTFONT].dat;
                        board_bmp = create_bitmap(320, 320);

                        /* Initilize the chess engine */   
                        playgame_init();
                        //ponder_init();
                        calculate_move_tables();

                        /* Initialise the hashtable. The number of entries
                         * should be a power-of-two.
                         */
                        initialize_hashtable(1<<9);
                        initialize_hashgen(1<<9);

                        sgenrand(time(NULL));

                        start_newgame(&game);
                        move_gen_start(&game.board, game.player, &game.mgl);
                        player = game.player;

                        quit = FALSE;
                        redraw_screen = TRUE;
                        select_state = SELECT_FREE;
                        move_input = FALSE;

			return true;
		}

		void Deinit() {
		  // custom deinitialization goes here
		  // ...
                  end_game(&game);
                  destroy_hashtable();
                  unload_datafile(chessgfx);   
                  destroy_bitmap(board_bmp);

                  remove_int(inc_sec_counter);

		  // no more Allegro calls after this line!
		  IGame::Deinit();
		}

		void Draw(BITMAP *canvas) {
                  draw_screen(canvas);
		}

		int Logic(int keys[8], int mouse[4], float dT) {
			// Do all input processing and game logic here. Refer to the
			// comments in IGame.h for details on the format of the input
			// parameters and the requirements for the output.
			// ...

                       /* We have a move to try */
                       if (move_input) {
                          from = from_x + ((7-from_y)<<3);
                          to = to_x + ((7-to_y)<<3);
                          piece = get_piece(&(game.board), from);
                          ptaken = get_piece(&(game.board), to);
                          /* Castling move? - but we don't know if it's allowed or not */
                          if ((piece&(KING|INIBIT)) == (KING|INIBIT) && abs(from_x-to_x)==2) {
                             piece |= CASTLE_MOVE;
                          }
                          /* En-passant */
                          /* The difficulty is that the square holding the pawn is stored in */
                          /*  the move state, while the destination is entered by the user */
                          if (piece&PAWN && abs(to-game.board.ep_square)==8 && abs(from-game.board.ep_square)==1) {
                             piece |= ENPASSANT_MOVE;
                             ptaken = get_piece(&(game.board), game.board.ep_square);
                          }
                          /* Promotion moves */
                          if (piece&PAWN && (to>H7 || to<A2)) {
                             /* Promotion - do the easy thing and take a Queen for now */
                             piece|=QUEEN;
                          }
                          encode_move(&m, piece, from, to, ptaken);
                          /* Has the user input a valid move? */
                          if (is_valid_move(m, &(game.mgl))) {
                             /* Perform the move, then make sure we're not being checked */
                             /*  because of it */
                             forward_move(&(game.board), m);
                             if (!check(&game.board, game.player)) {
                                backward_move(&(game.board), m);
                                playmove(&game, m);
                                move_gen_start(&game.board, game.player, &game.mgl);
                             } else {
                                backward_move(&(game.board), m);
                                printf("\a"); fflush(stdout);
                             }
                             redraw_screen = TRUE;
                          } else {
                             printf("\a"); fflush(stdout);
                          }
                          move_input = FALSE;
                       }

                       if (player != game.player) {   /* Input a move by the computer */
                           /* Have the computer play a move, with ply 1 search */
                           //stop_ponder();
                           sec_counter = -1;
                           abort_evaluation = 0;
                           while (sec_counter);
                           computer_play(&game, max_depth);
                           think_time = sec_counter;

                           display_prune_count = prune_count;
                           display_depth_reached = depth_reached;
                           display_evaluate_count = evaluate_count;

                           /* Have the computer ponder about the current position */
                           //ponder(&game);

                           /* Generate a new list of moves for the player, and set the player */
                           /*  to the one currently to move. This prevents an endless loop */
                           /* when the computer can't play any legal moves at all. */
                           move_gen_start(&game.board, game.player, &game.mgl);
                           player = game.player;
                           redraw_screen = TRUE;
                        } else {             /* Input a move by the player */
                          if (mouse_x>64 && mouse_y>64 && mouse_x<384 && mouse_y<384) {
                             int x = (mouse_x-64)/40;
                             int y = (mouse_y-64)/40;
                             if ((old_x!=x) || (old_y!=y)) {
                                old_x = sel_x = x;
                                old_y = sel_y = y;
                                redraw_screen = TRUE;
                             }
                             if (mouse_b&1) {
                                if (select_state == SELECT_FREE) {
                                   select_state = SELECT_ORIGIN;
                                   from_x = x;
                                   from_y = y;
                                   redraw_screen = TRUE;
                                } else if (select_state == SELECT_ORIGIN) {
                                   if (from_x!=x || from_y!=y) {
                                      select_state = SELECT_DONE;
                                      to_x = x;
                                      to_y = y;
                                      redraw_screen = TRUE;
                                      move_input = TRUE;
                                   }
                                }
                             } else {
                                if (select_state == SELECT_DONE) {
                                   select_state = SELECT_FREE;
                                   redraw_screen = TRUE;
                                }
                             }
                             if (mouse_b&2) {
                                select_state = SELECT_FREE;
                                redraw_screen = TRUE;
                             }
                          } else {
                             if (!(mouse_b&1)) {
                                select_state = SELECT_FREE;
                             }
                          }
                        }
			return quit;
		}

		int Score() {
			// Return -1 if this game doesn't have points, otherwise the last
			// valid score where more points is better.
			return -1;
		}

		bool UseSystemMouse() {
			// Return false if the game draws its own cursor or doesn't use
			// the mouse at all.
			return true;
		}

		void Pause() {
		}

		void Resume() {
		}

		// Should return true if keys can be redefined (otherwise
		// global keys are used)
		bool RedefinesKeys() {
			return false;
		}

		// Should return names of redefinable keys or NULL if
		// key can't be redefined. i = [0,7]
		const char *KeyName(int i) {
			switch (i) {
				default: return 0;
			};
		}

		// Should return to a NULL terminated array of GameSettings structures.
		// For example you could define the array like this:
		//
		//	GameSettings my_settings[] = {
		//		{ "name1", "option 1", "first|second|third", 1 },
		//		{ "name2", "option 2", "first|second|third", 0 },
		//		{ "name3", "option 3", "first|second|third", 2 },
		//		{ 0, 0, 0, 0 }
		//	};
		//
		// and then return my_settings in this function.
		GameSettings* GetSettings() {
			return 0;
		}
};



extern "C" DLLEXPORT IGame* GetPlugin() {
	return new ChessInterface;
}
