#include "ponder.h"
#include "compplay.h"
#include "minimax.h"
#include "alphabet.h"
#include "mtdf.h"
#include "genrand.h"
#include "evaluate.h"
#include "hashtab.h"
#include "movegen.h"
#include "square.h"
#include "game.h"

#ifdef USE_PTHREADS
   #ifdef unix
      #include <sys/resource.h>
   #endif
   #include <pthread.h>
#endif

static int pondering = 0;
static int ponder_on = 0;
static GAMESTATE ponder_state;

#ifdef USE_PTHREADS
static pthread_t ponder_thread;
static pthread_cond_t ponder_on_cond = PTHREAD_COND_INITIALIZER;;
static pthread_mutex_t ponder_on_lock = PTHREAD_MUTEX_INITIALIZER;;
static pthread_mutex_t ponder_lock = PTHREAD_MUTEX_INITIALIZER;;
#endif

static void *ponder_tread_function(void *args)
{
   int continue_pondering;
   
   
   #ifdef USE_PTHREADS
   #ifdef unix
      setpriority(PRIO_PROCESS, 0, 15);
   #endif
   #endif
   
   while (1) {

      #ifdef USE_PTHREADS
      pthread_mutex_lock(&ponder_lock);
      #endif
      pondering = 0;   
      #ifdef USE_PTHREADS
      pthread_mutex_unlock(&ponder_lock);
      #endif
      
      #ifdef USE_PTHREADS
      pthread_mutex_lock(&ponder_on_lock);
         while(!ponder_on) {
            //printf("Going to sleep...\n");
            pthread_cond_wait(&ponder_on_cond, &ponder_on_lock);
            //printf("Waking up\n");
         }
      pthread_mutex_unlock(&ponder_on_lock);
      #endif

      #ifdef USE_PTHREADS
      pthread_mutex_lock(&ponder_lock);
      #endif
      pondering = 1;
      #ifdef USE_PTHREADS
      pthread_mutex_unlock(&ponder_lock);
      #endif
      
      //printf("Pondering\n");
      
      computer_play(&ponder_state, 24);

      do {
         #ifdef USE_PTHREADS
            pthread_mutex_lock(&ponder_on_lock);
         #endif
         continue_pondering = ponder_on;
         #ifdef USE_PTHREADS
            pthread_mutex_unlock(&ponder_on_lock);
         #endif
         
      } while(continue_pondering);
   }
}

void ponder_init(void)
{
   ponder_on = 0;
   #ifdef USE_PTHREADS
      pthread_create(&ponder_thread, NULL, ponder_tread_function, NULL);
   #endif
}

void ponder(GAMESTATE *game)
{
   LINK *l;
   
   ponder_state = *game;
   
   ponder_state.movelist_tail = ponder_state.movelist = duplicate_list(game->movelist);
   for (l=ponder_state.movelist; l; l = l->next) {
      ponder_state.movelist_tail = l;
   }

   #ifdef USE_PTHREADS
      pthread_mutex_lock(&ponder_on_lock);
      ponder_on = 1;
      pthread_mutex_unlock(&ponder_on_lock);
      pthread_cond_broadcast(&ponder_on_cond);
   #endif
}

void stop_ponder(void)
{
   int waiting;
   abort_evaluation = 1;
   #ifdef USE_PTHREADS
      pthread_mutex_lock(&ponder_on_lock);
      ponder_on = 0;
      pthread_mutex_unlock(&ponder_on_lock);
      
      do {
         pthread_mutex_lock(&ponder_lock);
         waiting = pondering;   
         pthread_mutex_unlock(&ponder_lock);
      } while(waiting);
   #endif
}
