#include "hashgen.h"
#include "genrand.h"
#include "pieces.h"
#include "board.h"
#include "move.h"
#include "sbr.h"

typedef struct KEYGEN_STRUCT {
   int board_vals[BOARD_SQUARES][2*NUM_PIECE_TYPES];
   int player[2];
}  KEYGEN_STRUCT;

KEYGEN_STRUCT key1;
KEYGEN_STRUCT key2;

void initialize_hashgen(int tablesize)
{
   int c;
   int n;
   int u;
   int mask = (1<<sbr(tablesize))-1;
   
   u = 32 - sbr(tablesize);
   
   /* Initialize random numbers for each piece on each square */
   for (n=0; n<2*NUM_PIECE_TYPES; n++) {
      for (c=0; c<BOARD_SQUARES; c++) {
         key1.board_vals[c][n] = (genrandui()>>u) & mask;
         key2.board_vals[c][n] = (genrandui()>>u) & mask;
      }
   }

   /* Player to move */
   for (c=0; c<2; c++) {
      key1.player[c] = (genrandui()>>u) & mask;
      key2.player[c] = (genrandui()>>u) & mask;
   }
}

unsigned int calculate_key1(BOARD *board, int player) 
{ 
   int key = 0; 
   int c; 
   int n; 
   int p; 

   for (c=0;c<BOARD_SQUARES;c++) { 
      p = get_piece(board, c);
      if (p) {
         n = sbr(p&PIECE); 
         if (p&BLACK) 
            n+=NUM_PIECE_TYPES; 
         key ^= key1.board_vals[c][n];
      } 
   }

   key ^= key1.player[(player&BLACK)>>7]; 
   return key;
}

unsigned int update_key1(MOVE move, int key)
{ 
   int c; 
   int n;
   int p; 

   /* Switch player */
   key ^= key1.player[(get_move_player(move)&BLACK)>>7]; 
   key ^= key1.player[1-((get_move_player(move)&BLACK)>>7)]; 

   /* XOR the captured piece */
   p = get_captured_piece(move);
   if (p) {
      c = get_destination(move);
      n = sbr(p&PIECE); 
      if (p&BLACK) 
         n+=NUM_PIECE_TYPES; 
      key ^= key1.board_vals[c][n];
   } 
   
   /* XOR the moving piece */
   p = get_move_piece(move);
   n = sbr(p&PIECE); 
   if (p&BLACK) 
      n+=NUM_PIECE_TYPES; 

   c = get_origin(move);
   key ^= key1.board_vals[c][n];

   c = get_destination(move);
   key ^= key1.board_vals[c][n];
   
   return key;
}


unsigned int calculate_key2(BOARD *board, int player) 
{ 
   int key = 0; 
   int c; 
   int n; 
   int p; 

   for (c=0;c<BOARD_SQUARES;c++) { 
      p = get_piece(board, c);
      if (p) {
         n = sbr(p&PIECE); 
         if (p&BLACK) 
            n+=NUM_PIECE_TYPES; 
         key ^= key2.board_vals[c][n];
      } 
   }

   key ^= key2.player[(player&BLACK)>>7]; 
   return key;
}

unsigned int update_key2(MOVE move, int key)
{ 
   int c; 
   int n;
   int p; 

   /* Switch player */
   key ^= key2.player[(get_move_player(move)&BLACK)>>7]; 
   key ^= key2.player[1-((get_move_player(move)&BLACK)>>7)]; 

   /* XOR the captured piece */
   p = get_captured_piece(move);
   if (p) {
      c = get_destination(move);
      n = sbr(p&PIECE); 
      if (p&BLACK) 
         n+=NUM_PIECE_TYPES; 
      key ^= key2.board_vals[c][n];
   } 
   
   /* XOR the moving piece */
   p = get_move_piece(move);
   c = get_origin(move);
   n = sbr(p&PIECE); 
   if (p&BLACK) 
      n+=NUM_PIECE_TYPES; 
   key ^= key2.board_vals[c][n];
   c = get_destination(move);
   key ^= key2.board_vals[c][n];
   
   return key;
}
