#include <string.h>
#include "itype.h"
#include "board.h"
#include "pieces.h"

void clear_board(BOARD *board)
{
   memset(board, 0, sizeof *board);
}

void begin_board(BOARD *board)
{
   int c;
   
   clear_board(board);

   for (c=0; c<8; c++) {
      place_piece(board, A2+c, PAWN|WHITE|INIBIT);
      place_piece(board, A7+c, PAWN|BLACK|INIBIT);
   }
   place_piece(board, A1, ROOK|WHITE|INIBIT);
   place_piece(board, H1, ROOK|WHITE|INIBIT);
   place_piece(board, A8, ROOK|BLACK|INIBIT);
   place_piece(board, H8, ROOK|BLACK|INIBIT);

   place_piece(board, B1, KNIGHT|WHITE|INIBIT);
   place_piece(board, G1, KNIGHT|WHITE|INIBIT);
   place_piece(board, B8, KNIGHT|BLACK|INIBIT);
   place_piece(board, G8, KNIGHT|BLACK|INIBIT);

   place_piece(board, C1, BISHOP|WHITE|INIBIT);
   place_piece(board, F1, BISHOP|WHITE|INIBIT);
   place_piece(board, C8, BISHOP|BLACK|INIBIT);
   place_piece(board, F8, BISHOP|BLACK|INIBIT);

   place_piece(board, E1, KING|WHITE|INIBIT);
   place_piece(board, E8, KING|BLACK|INIBIT);

   place_piece(board, D1, QUEEN|WHITE|INIBIT);
   place_piece(board, D8, QUEEN|BLACK|INIBIT);
}


void calculate_bitboards(BOARD *board)
{
   int c;
   int p;
   
   for(c=0; c<7; c++)
      board->bbp[c] = 0;

   for(c=0; c<2; c++)
      board->bbc[c] = 0;
      
   for(c=0; c<64; c++) {
      p = get_piece(board, c);
      if (p) {
         /* Set colour */
         set_bitboard(&(board->bbc[(p>>7)&1]), c);
         if (p&INIBIT)
            set_bitboard(&(board->bbp[6]), c);
         set_bitboard(&(board->bbp[sbr_longlong(p&PIECE)]), c);
      }
   }
}

static void print_uint8(uint8 b)
{
   int c;
   
   for (c=0; c<8; c++) {
      printf ("%d", (b>>c)&1);
   }
}

void print_bitboards(BOARD *board)
{
   int c;

   printf("White pieces\tBlack pieces\tUnmoved pieces\n");
   for (c=7; c>=0; c--) {
      print_uint8(get_bitboard_row(board->bbc[0], c));
      printf("\t");
      print_uint8(get_bitboard_row(board->bbc[1], c));
      printf("\t");
      print_uint8(get_bitboard_row(board->bbp[6], c));
      printf("\n");
   }

   printf("\nPawns     Knights   Bishops   Rooks     Queens    Kings\n");
   for (c=7; c>=0; c--) {
      print_uint8(get_bitboard_row(board->bbp[0], c));
      printf("  ");
      print_uint8(get_bitboard_row(board->bbp[1], c));
      printf("  ");
      print_uint8(get_bitboard_row(board->bbp[2], c));
      printf("  ");
      print_uint8(get_bitboard_row(board->bbp[3], c));
      printf("  ");
      print_uint8(get_bitboard_row(board->bbp[4], c));
      printf("  ");
      print_uint8(get_bitboard_row(board->bbp[5], c));
      printf("\n");
   }

   /*
   printf("\nBoard-a   Board-h   Centre    X-centre  K-half    Q-half\n");
   for (c=7; c>=0; c--) {
      print_uint8(get_bitboard_row(board_minus_a, c));
      printf("  ");
      print_uint8(get_bitboard_row(board_minus_h, c));
      printf("  ");
      print_uint8(get_bitboard_row(board_centre, c));
      printf("  ");
      print_uint8(get_bitboard_row(board_xcentre, c));
      printf("  ");
      print_uint8(get_bitboard_row(board_kwing, c));
      printf("  ");
      print_uint8(get_bitboard_row(board_qwing, c));
      printf("\n");
   }

   printf("\n1strank   2ndrank   7thrank   8thrank                   \n");
   for (c=7; c>=0; c--) {
      print_uint8(get_bitboard_row(board_rank1, c));
      printf("  ");
      print_uint8(get_bitboard_row(board_rank2, c));
      printf("  ");
      print_uint8(get_bitboard_row(board_rank7, c));
      printf("  ");
      print_uint8(get_bitboard_row(board_rank8, c));
      printf("  ");
      print_uint8(get_bitboard_row(0, c));
      printf("  ");
      print_uint8(get_bitboard_row(0, c));
      printf("\n");
   }
   */
}
