#include "minimax.h"
#include "evaluate.h"
#include "game.h"

int minimax(GAMESTATE *game, const int plies)
{
   MOVE_GEN_STRUCT movelist;
   MOVE move;
   int valid_moves;
   int best_value;
   int value;
   
   if (plies==0) {   /* Reached search boundary (quiscence search) */
      return evaluate(game);
   }

   /* Setup semi-sensible default values */
   best_value = -INFINITY;
   value = -INFINITY;
   
   /* Get a list of valid moves in this position */   
   move_gen_start(&game->board, game->player, &movelist);
   /* Begin looking at each move in turn - if there are any, ofcourse */
   valid_moves = movelist.total.nummoves;

   while (get_next_move(&movelist, &move)) {
      playmove(game, move);
      /* Make sure we don't put ourselves in check */
      if (!check(&game->board, game->player^BLACK))
         value = -minimax(game, plies-1);
      else
         valid_moves--;
      takeback(game);
      
      if (value>best_value)
         best_value = value;
   }
   
   /* If there are no legal moves, we either have a mate or a stalemate */      
   if (!valid_moves) {
      return -CHECKMATE;   /* Checkmate */
   }
   
   //printf("\t%d %d\n", valid_moves, best_value);
   
   return best_value;
}
